from pypif.util.serializable import Serializable
from citrination_client.search.dataset.result.dataset_search_result import DatasetSearchResult


class DatasetMultiSearchResult(Serializable):
    """
    Class to store the results of a dataset multi-query.
    """

    def __init__(self, took=None, results=None, **kwargs):
        """
        Constructor.

        :param took: Number of milliseconds that the query took to execute.
        :param results: List of :class:`.DatasetSearchResult` objects.
        """
        self._took = None
        self.took = took
        self._results = None
        self.results = results

    @property
    def took(self):
        return self._took

    @took.setter
    def took(self, took):
        self._took = took

    @took.deleter
    def took(self):
        self._took = None

    @property
    def results(self):
        return self._results

    @results.setter
    def results(self, results):
        self._results = self._get_object(DatasetSearchResult, results)

    @results.deleter
    def results(self):
        self._results = None



from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery
from citrination_client.search.pif.query.core.value_query import ValueQuery


class ProcessStepQuery(BaseObjectQuery):
    """
    Class to query against a process step.
    """

    def __init__(self, name=None, details=None, logic=None, extract_as=None, extract_all=None,
                 extract_when_missing=None, tags=None, length=None, offset=None):
        """
        Constructor.

        :param name: One or more :class:`FieldQuery` operations against the name field.
        :param details: One or more :class:`ValueQuery` operations against the details of the step.
        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        """
        super(ProcessStepQuery, self).__init__(logic=logic, extract_as=extract_as, extract_all=extract_all,
                                               extract_when_missing=extract_when_missing, tags=tags, length=length,
                                               offset=offset)
        self._name = None
        self.name = name
        self._details = None
        self.details = details

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = self._get_object(FieldQuery, name)

    @name.deleter
    def name(self):
        self._name = None

    @property
    def details(self):
        return self._details

    @details.setter
    def details(self, details):
        self._details = self._get_object(ValueQuery, details)

    @details.deleter
    def details(self):
        self._details = None



from citrination_client.search.pif.query.chemical.chemical_filter import ChemicalFilter
from citrination_client.search.pif.query.core.base_field_query import BaseFieldQuery


class ChemicalFieldQuery(BaseFieldQuery):
    """
    Class for all field operations against chemical information.
    """

    def __init__(self, filter=None, logic=None, extract_as=None, extract_all=None, extract_when_missing=None,
                 length=None, offset=None):
        """
        Constructor.

        :param filter: One or more :class:`.ChemicalFilter` objects for the query.
        :param logic: Logic for this query. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param length: One or more :class:`.FieldOperation` objects against the length field.
        :param offset: One or more :class:`.FieldOperation` objects against the offset field.
        :param filter: One or more :class:`.ChemicalFilter` objects against this field.
        """
        super(ChemicalFieldQuery, self).__init__(logic=logic, extract_as=extract_as, extract_all=extract_all,
                                                 extract_when_missing=extract_when_missing, length=length,
                                                 offset=offset)
        self._filter = None
        self.filter = filter

    @property
    def filter(self):
        return self._filter

    @filter.setter
    def filter(self, filter):
        self._filter = self._get_object(ChemicalFilter, filter)

    @filter.deleter
    def filter(self):
        self._filter = None



from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery
from citrination_client.search.pif.query.core.file_reference_query import FileReferenceQuery
from citrination_client.search.pif.query.core.units_normalization import UnitsNormalization


class ValueQuery(BaseObjectQuery):
    """
    Class to query against a single value.
    """

    def __init__(self, name=None, value=None, file=None, units=None, units_normalization=None, logic=None,
                 extract_as=None, extract_all=None, extract_when_missing=None, tags=None, length=None, offset=None):
        """
        Constructor.

        :param name: One or more :class:`FieldQuery` operations against the name field.
        :param value: One or more :class:`FieldQuery` operations against the value.
        :param file: One or more :class:`FileReferenceQuery` operations against the file.
        :param units: One or more :class:`FieldQuery` operations against the units field.
        :param units_normalization: :class:`UnitsNormalization` object for normalizing units.
        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        """
        super(ValueQuery, self).__init__(logic=logic, extract_as=extract_as, extract_all=extract_all,
                                         extract_when_missing=extract_when_missing, tags=tags,
                                         length=length, offset=offset)
        self._name = None
        self.name = name
        self._value = None
        self.value = value
        self._file = None
        self.file = file
        self._units = None
        self.units = units
        self._units_normalization = None
        self.units_normalization = units_normalization

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = self._get_object(FieldQuery, name)

    @name.deleter
    def name(self):
        self._name = None

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = self._get_object(FieldQuery, value)

    @value.deleter
    def value(self):
        self._value = None

    @property
    def file(self):
        return self._file

    @file.setter
    def file(self, file):
        self._file = self._get_object(FileReferenceQuery, file)

    @file.deleter
    def file(self):
        self._file = None

    @property
    def units(self):
        return self._units

    @units.setter
    def units(self, units):
        self._units = self._get_object(FieldQuery, units)

    @units.deleter
    def units(self):
        self._units = None

    @property
    def units_normalization(self):
        return self._units_normalization

    @units_normalization.setter
    def units_normalization(self, units_normalization):
        self._units_normalization = self._get_object(UnitsNormalization, units_normalization)

    @units_normalization.deleter
    def units_normalization(self):
        self._units_normalization = None



from citrination_client.search.pif.query.core.field_query import FieldQuery


def FieldOperation(filter=None, extract_as=None, extract_all=None, extract_when_missing=None,
                   length=None, offset=None):
    """
    Generate a new :class:`.FieldQuery` object.

    :param filter: One or more :class:`.Filter` objects for the query.
    :param extract_as: String with the alias to save this field under.
    :param extract_all: Boolean setting whether all values in an array should be extracted.
    :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
    is missing that should be extracted (and the overall query is still satisfied).
    :param length: One or more :class:`.FieldOperation` objects against the length field.
    :param offset: One or more :class:`.FieldOperation` objects against the offset field.
    :param filter: One or more :class:`.Filter` objects against this field.
    """
    from warnings import warn
    warn("FieldOperation has been deprecated in favor of FieldQuery starting with version 1.3.0")
    return FieldQuery(filter=filter, extract_as=extract_as, extract_all=extract_all,
                      extract_when_missing=extract_when_missing, length=length, offset=offset)



from citrination_client.search.pif.query.pif_query import PifQuery
from citrination_client.search.pif.query.core.classification_query import ClassificationQuery
from citrination_client.search.pif.query.core.field_operation import FieldOperation
from citrination_client.search.pif.query.core.field_query import FieldQuery
from citrination_client.search.pif.query.core.file_reference_query import FileReferenceQuery
from citrination_client.search.pif.query.core.filter import Filter
from citrination_client.search.pif.query.core.id_query import IdQuery
from citrination_client.search.pif.query.core.name_query import NameQuery
from citrination_client.search.pif.query.core.pages_query import PagesQuery
from citrination_client.search.pif.query.core.process_step_query import ProcessStepQuery
from citrination_client.search.pif.query.core.property_query import PropertyQuery
from citrination_client.search.pif.query.core.quantity_query import QuantityQuery
from citrination_client.search.pif.query.core.reference_query import ReferenceQuery
from citrination_client.search.pif.query.core.sort_extracted import SortExtracted
from citrination_client.search.pif.query.core.source_query import SourceQuery
from citrination_client.search.pif.query.core.system_query import SystemQuery
from citrination_client.search.pif.query.core.units_normalization import UnitsNormalization
from citrination_client.search.pif.query.core.value_query import ValueQuery
from citrination_client.search.pif.query.chemical.chemical_field_operation import ChemicalFieldOperation
from citrination_client.search.pif.query.chemical.chemical_field_query import ChemicalFieldQuery
from citrination_client.search.pif.query.chemical.chemical_filter import ChemicalFilter
from citrination_client.search.pif.query.chemical.composition_query import CompositionQuery



from pypif.util.serializable import Serializable


class ChemicalFilter(Serializable):
    """
    Filter that can be applied to a field that stores chemical information.
    """

    def __init__(self, logic=None, exists=None, equal=None, element=None, partial=None, exact=None, filter=None):
        """
        Constructor.

        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param exists: True/False to simply test whether the field exists and has a non-null value.
        :param equal: String with the phrase to match against.
        :param element: True to match against the single element field.
        :param partial: True to match against the partial formula field.
        :param exact: True if matches should be exact.
        :param filter: List of :class:`ChemicalFilter` objects with sub-filters.
        """
        self._logic = None
        self.logic = logic
        self._exists = None
        self.exists = exists
        self._equal = None
        self.equal = equal
        self._element = None
        self.element = element
        self._partial = None
        self.partial = partial
        self._exact = None
        self.exact = exact
        self._filter = None
        self.filter = filter

    @property
    def logic(self):
        return self._logic

    @logic.setter
    def logic(self, logic):
        self._logic = logic

    @logic.deleter
    def logic(self):
        self._logic = None

    @property
    def exists(self):
        return self._exists

    @exists.setter
    def exists(self, exists):
        self._exists = exists

    @exists.deleter
    def exists(self):
        self._exists = None

    @property
    def equal(self):
        return self._equal

    @equal.setter
    def equal(self, equal):
        self._equal = equal

    @equal.deleter
    def equal(self):
        self._equal = None

    @property
    def element(self):
        return self._element

    @element.setter
    def element(self, element):
        self._element = element

    @element.deleter
    def element(self):
        self._element = None

    @property
    def partial(self):
        return self._partial

    @partial.setter
    def partial(self, partial):
        self._partial = partial

    @partial.deleter
    def partial(self):
        self._partial = None

    @property
    def exact(self):
        return self._exact

    @exact.setter
    def exact(self, exact):
        self._exact = exact

    @exact.deleter
    def exact(self):
        self._exact = None

    @property
    def filter(self):
        return self._filter

    @filter.setter
    def filter(self, filter):
        self._filter = self._get_object(ChemicalFilter, filter)

    @filter.deleter
    def filter(self):
        self._filter = None



from citrination_client.search.pif.result.pif_search_hit import PifSearchHit
from citrination_client.search.pif.result.pif_search_result import PifSearchResult
from citrination_client.search.pif.result.pif_multi_search_result import PifMultiSearchResult



from citrination_client.search.dataset.result.dataset_search_hit import DatasetSearchHit
from citrination_client.search.dataset.result.dataset_search_result import DatasetSearchResult
from citrination_client.search.dataset.result.dataset_multi_search_result import DatasetMultiSearchResult



from pypif.util.serializable import Serializable


class DatasetSearchHit(Serializable):
    """
    Class to store a single dataset search hit.
    """

    def __init__(self, id=None, score=None, num_pifs=None, **kwargs):
        """
        Constructor.

        :param id: String with the ID of the record.
        :param score: Score with the relevancy of the result.
        :param num_pifs: Number of PIFs in the dataset.
        """
        self._id = None
        self.id = id
        self._score = None
        self.score = score
        self._num_pifs = None
        self.num_pifs = num_pifs

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @id.deleter
    def id(self):
        self._id = None

    @property
    def score(self):
        return self._score

    @score.setter
    def score(self, score):
        self._score = score

    @score.deleter
    def score(self):
        self._score = None

    @property
    def num_pifs(self):
        return self._num_pifs

    @num_pifs.setter
    def num_pifs(self, num_pifs):
        self._num_pifs = num_pifs

    @num_pifs.deleter
    def num_pifs(self):
        self._num_pifs = None



from six import string_types
from pypif import pif
from pypif.obj.common.pio import Pio
from pypif.util.serializable import Serializable


class PifSearchHit(Serializable):
    """
    Class to store a single PIF search hit.
    """

    def __init__(self, id=None, dataset=None, dataset_version=None, score=None, system=None, extracted=None, **kwargs):
        """
        Constructor.

        :param id: String with the ID of the record.
        :param dataset: Integer with the dataset of the record.
        :param dataset_version: Integer with the dataset version of the record.
        :param score: Score with the relevancy of the result.
        :param system: Pif System object that matched.
        :param extracted: Dictionary with a map of extracted property names to values.
        """
        self._id = None
        self.id = id
        self._dataset = None
        self.dataset = dataset
        self._dataset_version = None
        self.dataset_version = dataset_version
        self._score = None
        self.score = score
        self._system = None
        self.system = system
        self._extracted = None
        self.extracted = extracted

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id

    @id.deleter
    def id(self):
        self._id = None

    @property
    def dataset(self):
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        self._dataset = dataset

    @dataset.deleter
    def dataset(self):
        self._dataset = None

    @property
    def dataset_version(self):
        return self._dataset_version

    @dataset_version.setter
    def dataset_version(self, dataset_version):
        self._dataset_version = dataset_version

    @dataset_version.deleter
    def dataset_version(self):
        self._dataset_version = None

    @property
    def score(self):
        return self._score

    @score.setter
    def score(self, score):
        self._score = score

    @score.deleter
    def score(self):
        self._score = None

    @property
    def system(self):
        return self._system

    @system.setter
    def system(self, system):
        if system is None:
            self._system = None
        elif isinstance(system, string_types):
            self._system = pif.loads(system)
        elif isinstance(system, dict):
            self._system = pif.loado(system)
        elif isinstance(system, Pio):
            self._system = system
        else:
            raise TypeError('Not a valid system type: must be string, dict, or Pio, but got ' + str(type(system)))

    @system.deleter
    def system(self):
        self._system = None

    @property
    def extracted(self):
        return self._extracted

    @extracted.setter
    def extracted(self, extracted):
        self._extracted = extracted

    @extracted.deleter
    def extracted(self):
        self._extracted = None



from pypif.util.serializable import Serializable
from citrination_client.search.pif.query.pif_query import PifQuery


class PifMultiQuery(Serializable):
    """
    Class to store information about a PIF multi-query.
    """

    def __init__(self, queries=None):
        """
        Constructor.

        :param queries: One or more :class:`.PifQuery` objects with the queries to run.
        """
        self._queries = None
        self.queries = queries

    @property
    def queries(self):
        return self._queries

    @queries.setter
    def queries(self, queries):
        self._queries = self._get_object(PifQuery, queries)

    @queries.deleter
    def queries(self):
        self._queries = None



from pypif.util.serializable import Serializable


class UnitsNormalization(Serializable):
    """
    Configuration for normalization of units.
    """

    def __init__(self, search=None, extract=None):
        """
        Constructor.

        :param search: String with the units to run the query against.
        :param extract: String with the units to normalize results to.
        """
        self._search = None
        self.search = search
        self._extract = None
        self.extract = extract

    @property
    def search(self):
        return self._search

    @search.setter
    def search(self, search):
        self._search = search

    @search.deleter
    def search(self):
        self._search = None

    @property
    def extract(self):
        return self._extract

    @extract.setter
    def extract(self, extract):
        self._extract = extract

    @extract.deleter
    def extract(self):
        self._extract = None



from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
from citrination_client.search.pif.query.core.classification_query import ClassificationQuery
from citrination_client.search.pif.query.core.field_query import FieldQuery
from citrination_client.search.pif.query.core.id_query import IdQuery
from citrination_client.search.pif.query.core.process_step_query import ProcessStepQuery
from citrination_client.search.pif.query.core.property_query import PropertyQuery
from citrination_client.search.pif.query.core.quantity_query import QuantityQuery
from citrination_client.search.pif.query.core.reference_query import ReferenceQuery
from citrination_client.search.pif.query.core.source_query import SourceQuery
from citrination_client.search.pif.query.chemical.chemical_field_query import ChemicalFieldQuery
from citrination_client.search.pif.query.chemical.composition_query import CompositionQuery


class SystemQuery(BaseObjectQuery):
    """
    Class to query against a PIF System.
    """

    def __init__(self, names=None, ids=None, classifications=None, source=None, quantity=None, chemical_formula=None, 
                 composition=None, properties=None, preparation=None, references=None, sub_systems=None, logic=None, 
                 extract_as=None, extract_all=None, extract_when_missing=None, tags=None, length=None, offset=None):
        """
        Constructor.

        :param names: One or more :class:`FieldQuery` operations against the names field.
        :param ids: One or more :class:`IdQuery` operations against the ids field.
        :param classifications: One or more :class:`ClassificationQuery` operations against the classifications field.
        :param source: One or more :class:`SourceQuery` operations against the source field.
        :param quantity: One or more :class:`QuantityQuery` operations against the quantity field.
        :param chemical_formula: One or more :class:`ChemicalFieldQuery` operations against the
        chemical formula field.
        :param composition: One or more :class:`CompositionQuery` operations against the composition field.
        :param properties: One or more :class:`PropertyQuery` operations against the properties field.
        :param preparation: One or more :class:`ProcessStepQuery` operations against the preparation field.
        :param references: One or more :class:`ReferenceQuery` operations against the references field.
        :param sub_systems: One or more :class:`SystemQuery` operations against the sub systems field.
        :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
        :param extract_as: String with the alias to save this field under.
        :param extract_all: Boolean setting whether all values in an array should be extracted.
        :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
        is missing that should be extracted (and the overall query is still satisfied).
        :param tags: One or more :class:`FieldQuery` operations against the tags field.
        :param length: One or more :class:`FieldQuery` operations against the length field.
        :param offset: One or more :class:`FieldQuery` operations against the offset field.
        """
        super(SystemQuery, self).__init__(logic=logic, extract_as=extract_as, extract_all=extract_all,
                                          extract_when_missing=extract_when_missing, tags=tags, length=length,
                                          offset=offset)
        self._names = None
        self.names = names
        self._ids = None
        self.ids = ids
        self._classifications = None
        self.classifications = classifications
        self._source = None
        self.source = source
        self._quantity = None
        self.quantity = quantity
        self._chemical_formula = None
        self.chemical_formula = chemical_formula
        self._composition = None
        self.composition = composition
        self._properties = None
        self.properties = properties
        self._preparation = None
        self.preparation = preparation
        self._references = None
        self.references = references
        self._sub_systems = None
        self.sub_systems = sub_systems

    @property
    def names(self):
        return self._names

    @names.setter
    def names(self, names):
        self._names = self._get_object(FieldQuery, names)

    @names.deleter
    def names(self):
        self._names = None

    @property
    def ids(self):
        return self._ids

    @ids.setter
    def ids(self, ids):
        self._ids = self._get_object(IdQuery, ids)

    @ids.deleter
    def ids(self):
        self._ids = None

    @property
    def classifications(self):
        return self._classifications

    @classifications.setter
    def classifications(self, classifications):
        self._classifications = self._get_object(ClassificationQuery, classifications)

    @classifications.deleter
    def classifications(self):
        self._classifications = None

    @property
    def source(self):
        return self._source

    @source.setter
    def source(self, source):
        self._source = self._get_object(SourceQuery, source)

    @source.deleter
    def source(self):
        self._source = None

    @property
    def quantity(self):
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        self._quantity = self._get_object(QuantityQuery, quantity)

    @quantity.deleter
    def quantity(self):
        self._quantity = None

    @property
    def chemical_formula(self):
        return self._chemical_formula

    @chemical_formula.setter
    def chemical_formula(self, chemical_formula):
        self._chemical_formula = self._get_object(ChemicalFieldQuery, chemical_formula)

    @chemical_formula.deleter
    def chemical_formula(self):
        self._chemical_formula = None

    @property
    def composition(self):
        return self._composition

    @composition.setter
    def composition(self, composition):
        self._composition = self._get_object(CompositionQuery, composition)

    @composition.deleter
    def composition(self):
        self._composition = None

    @property
    def properties(self):
        return self._properties

    @properties.setter
    def properties(self, properties):
        self._properties = self._get_object(PropertyQuery, properties)

    @properties.deleter
    def properties(self):
        self._properties = None

    @property
    def preparation(self):
        return self._preparation

    @preparation.setter
    def preparation(self, preparation):
        self._preparation = self._get_object(ProcessStepQuery, preparation)

    @preparation.deleter
    def preparation(self):
        self._preparation = None

    @property
    def references(self):
        return self._references

    @references.setter
    def references(self, references):
        self._references = self._get_object(ReferenceQuery, references)

    @references.deleter
    def references(self):
        self._references = None

    @property
    def sub_systems(self):
        return self._sub_systems

    @sub_systems.setter
    def sub_systems(self, sub_systems):
        self._sub_systems = self._get_object(SystemQuery, sub_systems)

    @sub_systems.deleter
    def sub_systems(self):
        self._sub_systems = None



from citrination_client.search.pif.query.core.base_field_query import BaseFieldQuery


def BaseFieldOperation(extract_as=None, extract_all=None, extract_when_missing=None, length=None, offset=None):
    """
    Generate a new :class:`.BaseFieldQuery` object.

    :param extract_as: String with the alias to save this field under.
    :param extract_all: Boolean setting whether all values in an array should be extracted.
    :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
    is missing that should be extracted (and the overall query is still satisfied).
    :param length: One or more :class:`.FieldQuery` operations against the length field.
    :param offset: One or more :class:`.FieldQuery` operations against the offset field.
    """
    from warnings import warn
    warn("BaseFieldOperation has been deprecated in favor of BaseFieldQuery starting with version 1.3.0")
    return BaseFieldQuery(extract_as=extract_as, extract_all=extract_all, extract_when_missing=extract_when_missing,
                          length=length, offset=offset)
